@@ -8,6 +8,8 @@
 
 #pragma once
 
+#include "ChannelGroups.h"
+#include "data/EpgGenre.h"
 #include "data/MediaEntry.h"
 
 #include <string>
@@ -19,17 +21,19 @@ namespace iptvsimple
   class ATTR_DLL_LOCAL Media
   {
   public:
-    Media();
+    Media(std::shared_ptr<iptvsimple::InstanceSettings>& settings);
     void GetMedia(std::vector<kodi::addon::PVRRecording>& kodiRecordings);
     int GetNumMedia() const;
     void Clear();
     const std::string GetMediaEntryURL(const kodi::addon::PVRRecording& mediaEntry);
     const iptvsimple::data::MediaEntry* FindMediaEntry(const std::string& id, const std::string& displayName) const;
 
-    bool AddMediaEntry(iptvsimple::data::MediaEntry& entry);
+    bool AddMediaEntry(iptvsimple::data::MediaEntry& entry, std::vector<int>& groupIdList, iptvsimple::ChannelGroups& channelGroups, bool channelHadGroups);
 
     std::vector<iptvsimple::data::MediaEntry>& GetMediaEntryList() { return m_media; }
 
+    void SetGenreMappings(std::vector<iptvsimple::data::EpgGenre>& genreMappings) { m_genreMappings = genreMappings; }
+
   private:
     data::MediaEntry GetMediaEntry(const std::string& mediaEntryId) const;
     bool IsInVirtualMediaEntryFolder(const data::MediaEntry& mediaEntry) const;
@@ -37,6 +41,10 @@ namespace iptvsimple
     std::vector<iptvsimple::data::MediaEntry> m_media;
     std::unordered_map<std::string, iptvsimple::data::MediaEntry> m_mediaIdMap;
 
+    std::vector<iptvsimple::data::EpgGenre> m_genreMappings;
+
     bool m_haveMediaTypes = false;
+
+    std::shared_ptr<iptvsimple::InstanceSettings> m_settings;
   };
 } //namespace iptvsimple
